from django.db import models
import numpy as np

def find_segments_in_prediction_array(prediction_array: np.array, min_frame_len: int):
    """
    Expects a prediction array of shape (num_frames) and a minimum frame length.
    Returns a list of tuples (start_frame_number, end_frame_number) that represent the segments.
    """
    # Add False to the beginning and end to detect changes at the array boundaries
    padded_prediction = np.pad(prediction_array, (1, 1), 'constant', constant_values=False)
    
    # Find the start points and end points of the segments
    diffs = np.diff(padded_prediction.astype(int))
    segment_starts = np.where(diffs == 1)[0]
    segment_ends = np.where(diffs == -1)[0]
    
    # Filter segments based on min_frame_len
    segments = [(start, end) for start, end in zip(segment_starts, segment_ends) if end - start >= min_frame_len]
    
    return segments

class AbstractLabelVideoSegment(models.Model):
    video = None # Placeholder for the video field, to be defined in derived classes
    prediction_meta = None # Placeholder for the prediction_meta field, to be defined in derived classes
    start_frame_number = models.IntegerField()
    end_frame_number = models.IntegerField()
    source = models.ForeignKey("InformationSource", on_delete=models.CASCADE)
    label = models.ForeignKey("Label", on_delete=models.CASCADE)

    class Meta:
        abstract = True

    def __str__(self):
        return self.video.file.path + " Label - " + self.label.name + " - " + str(self.start_frame_number) + " - " + str(self.end_frame_number)
    
    def get_frames(self):
        return self.video.get_frame_range(self.start_frame_number, self.end_frame_number)

class LegacyLabelVideoSegment(AbstractLabelVideoSegment):
    video = models.ForeignKey("LegacyVideo", on_delete=models.CASCADE)
    prediction_meta = models.ForeignKey("LegacyVideoPredictionMeta", on_delete=models.CASCADE, related_name="video_segments")





from django.conf import settings
from django.core.management.base import BaseCommand
from reports.models import Intervention, InterventionType  # Replace 'your_app_name' with the actual app name
import os
from reports.dataloader_utils import load_model_data

SOURCE_DIR = None # e.g. settings.DATA_DIR_INTERVENTION

IMPORT_MODELS = [ # string as model key, serves as key in IMPORT_METADATA
]

IMPORT_METADATA = {
    # "": { # same as model name in "import models", e.g. "Intervention"
    #     "dir": os.path.join(SOURCE_DIR,""), # e.g. "interventions"
    #     "model": None, # e.g. Intervention
    #     "foreign_keys": [], # e.g. ["intervention_types"]
    #     "foreign_key_models": [] # e.g. [InterventionType]
    # },
}

class Command(BaseCommand):
    help = """Load all .yaml files in the data/intervention directory
    into the Intervention and InterventionType model"""

    def add_arguments(self, parser):
        parser.add_argument(
            '--verbose',
            action='store_true',
            help='Display verbose output',
        )

    def handle(self, *args, **options):
        verbose = options['verbose']
        for model_name in IMPORT_MODELS:
            _metadata = IMPORT_METADATA[model_name]
            load_model_data(
                self,
                model_name,
                _metadata,
                verbose
            )


from django.db import models

from agl_base_db.models.data_file.frame import Frame
from agl_base_db.models.data_file.frame import LegacyFrame

BATCH_SIZE = 1000
from .base_classes import AbstractVideo


class Video(AbstractVideo):
    def get_video_model(self):
        return Video
    def get_frame_model(self):
        return Frame
    

class LegacyVideo(AbstractVideo):
    file = models.FileField(upload_to="legacy_videos", blank=True, null=True)

    def get_video_model(self):
        return LegacyVideo
    def get_frame_model(self):
        return LegacyFrame

    